package de.mlessmann.certassist.keystore;

import de.mlessmann.certassist.except.JavaSecurityException;
import de.mlessmann.certassist.openssl.CertificatePasswordProvider;
import de.mlessmann.certassist.openssl.CertificateUsage;
import de.mlessmann.certassist.openssl.OpenSSLCertificateCreator;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.OpenOption;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.security.*;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Base64;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@RequiredArgsConstructor
public class KeyStoreManager {

    private static final OpenOption[] CREATE_TRUNCATE = {
        StandardOpenOption.CREATE,
        StandardOpenOption.TRUNCATE_EXISTING,
    };
    private final OpenSSLCertificateCreator certificateCreator;
    private final CertificatePasswordProvider passwordProvider;

    public KeystoreUsage createKeyStore(String keyStorePassphrase, CertificateUsage... serverCerts)
        throws JavaSecurityException {
        try {
            Path keystorePath = Files.createTempFile("keystore", ".jks");

            // Load the keystore
            KeyStore keystore = KeyStore.getInstance(KeyStore.getDefaultType());
            keystore.load(null, null);

            for (CertificateUsage serverCert : serverCerts) {
                PrivateKey privateKey = loadPrivateKey(
                    serverCert.certificateKeyPath(),
                    passwordProvider.getPasswordFor(serverCert.fingerprint())
                );
                Certificate[] certChain = loadCertificateChain(serverCert.fullchainPath());
                keystore.setKeyEntry(serverCert.fingerprint(), privateKey, null, certChain);
            }

            // Save the keystore
            try (var keystoreOut = Files.newOutputStream(keystorePath, CREATE_TRUNCATE)) {
                keystore.store(keystoreOut, keyStorePassphrase.toCharArray());
            }
            return new KeystoreResult(keystorePath);
        } catch (IOException | KeyStoreException | NoSuchAlgorithmException | CertificateException e) {
            throw new JavaSecurityException("Failed to create keystore!", e);
        }
    }

    @SneakyThrows
    private PrivateKey loadPrivateKey(Path privateKey, String passphrase) {
        String pemContent;
        if (certificateCreator.isKeyEncrypted(privateKey)) {
            pemContent = certificateCreator.readDecryptedKey(privateKey, passphrase);
        } else {
            pemContent = Files.readString(privateKey);
        }

        try (var fis = Files.newInputStream(privateKey)) {
            String privateKeyPEM = pemContent
                .replaceAll(".*?-----BEGIN PRIVATE KEY-----", "")
                .replace("-----END PRIVATE KEY-----", "")
                .replaceAll("\\s", "");
            byte[] decodedKey = Base64.getDecoder().decode(privateKeyPEM);
            return KeyFactory.getInstance("RSA").generatePrivate(new PKCS8EncodedKeySpec(decodedKey));
        }
    }

    @SneakyThrows
    private Certificate[] loadCertificateChain(Path certChainPath) {
        // Load the certificate chain from a PEM file
        try (var fis = Files.newInputStream(certChainPath)) {
            CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
            return certFactory.generateCertificates(fis).toArray(Certificate[]::new);
        }
    }
}
