package de.mlessmann.certassist.keystore;

import de.mlessmann.certassist.openssl.CertificateUsage;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.OpenOption;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@RequiredArgsConstructor
public class TruststoreManager {

    private static final OpenOption[] CREATE_TRUNCATE = {
        StandardOpenOption.CREATE,
        StandardOpenOption.TRUNCATE_EXISTING,
    };

    public KeystoreUsage createTruststore(String truststorePassphrase, CertificateUsage... trustedCertificates) {
        try {
            Path truststorePath = Files.createTempFile("truststore", ".jks");
            truststorePath.toFile().deleteOnExit();
            log.debug("Creating truststore at {}", truststorePath);

            KeyStore truststore = buildTruststore(trustedCertificates);
            try (var outputStream = Files.newOutputStream(truststorePath, CREATE_TRUNCATE)) {
                truststore.store(outputStream, truststorePassphrase.toCharArray());
            }
            return new KeystoreResult(truststorePath);
        } catch (IOException | KeyStoreException | CertificateException | NoSuchAlgorithmException e) {
            throw new IllegalStateException("Failed to create truststore!", e);
        }
    }

    @SneakyThrows
    private static KeyStore buildTruststore(CertificateUsage[] trustedCertificates) {
        KeyStore truststore = KeyStore.getInstance(KeyStore.getDefaultType());
        truststore.load(null, null);
        CertificateFactory certificateFactory = CertificateFactory.getInstance("X.509");

        if (trustedCertificates == null || trustedCertificates.length == 0) {
            log.warn("No trusted certificates provided, truststore will be empty!");
        } else {
            for (CertificateUsage trustedCertificate : trustedCertificates) {
                try (var inputStream = Files.newInputStream(trustedCertificate.certificatePath())) {
                    X509Certificate jdkCert = (X509Certificate) certificateFactory.generateCertificate(inputStream);
                    truststore.setCertificateEntry(trustedCertificate.fingerprint(), jdkCert);
                }
            }
        }
        return truststore;
    }
}
